using System;
using System.Collections.Generic;
using System.Text;
using Genesyslab.Platform.OpenMedia.Protocols;
using System.Threading;
using Genesyslab.Platform.Commons.Protocols;
using Genesyslab.Platform.OpenMedia.Protocols.InteractionServer;
using Genesyslab.Platform.Commons.Collections;
using Genesyslab.Platform.OpenMedia.Protocols.InteractionServer.Requests.AgentManagement;
using Genesyslab.Platform.OpenMedia.Protocols.InteractionServer.Events;
using Genesyslab.Platform.OpenMedia.Protocols.InteractionServer.Requests.InteractionDelivery;
using Genesyslab.Platform.OpenMedia.Protocols.InteractionServer.Requests.InteractionManagement;
using System.Windows.Forms;

namespace InteractionReceiver
{
    class Program
    {
        public TimeSpan timespan; 
        public InteractionServerProtocol itxServer;
        public Thread msgReceiver;
        
        public string host;
        public int port;
        public string employeeId;
        public string placeId;
        public int tenantId;
        public string mediaType;

        public bool AgentLoggedIn;
        public bool AgentReady;

        #region (1) Application initialization 

        //Main application block
        public Program()
        {
            //set timeout for response from interaction server
            timespan = new TimeSpan(0, 0, 1000);

            //interaction server connect parameters
            host = "hostname";
            port = 7002;

            //Agent login parameters
            employeeId = "id_10000";
            placeId = "p_10000";
            tenantId = 104;
            mediaType = "email";


            AgentLoggedIn = false;
            AgentReady = false;

            
            //Connect to interaction server
            if (!ConnectToItxServer())
                return;

            //Start message receiver thread
            msgReceiver = new Thread(new ThreadStart(ReceiveMessages));
            msgReceiver.Start();
        }

        #endregion

        #region (2) Connection / Disconnection from Interaction server

        //Connect to interaction server
        public bool ConnectToItxServer()
        {
            //Create endpoint for Interaction Server connection
            Endpoint itxServerEndpoint = new Endpoint("itxServer", host, port);
            //Create protocol for endpoint
            itxServer = new InteractionServerProtocol(itxServerEndpoint);
            //Specify protocol attributes
            itxServer.ClientType = InteractionClient.AgentApplication;
            itxServer.ClientName = "Platform SDK .NET Custom Interaction receiver";

            try
            {
                //Connect to Interaction Server
                itxServer.Open();
            }
            catch (ProtocolException ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine("Can't connect to interaction server");
                return false;
            }
            Console.WriteLine("Connected to interaction server...");
            return true;
        }

        //Disconnect from interaction server
        public bool DisonnectFromItxServer()
        {
            try
            {
                itxServer.Close();
                msgReceiver.Join();
            }
            catch (ProtocolException ex)
            {
                Console.WriteLine(ex.Message);
                return false;
            }
            Console.WriteLine("Disconnected from interaction server...");
            return true;
        }

        #endregion

        #region (3) Agent status manipulation. Login,Logout,Ready and AddMedia

        //Make agent ready
        public bool MakeAgentReady()
        {
            try
            {   // create request
                RequestCancelNotReadyForMedia requestCancelNotReadyForMedia =
                    RequestCancelNotReadyForMedia.Create(
                    mediaType,
                    null);

                Console.WriteLine(requestCancelNotReadyForMedia.ToString());
                // send request and receive response
                IMessage respondingEvent = itxServer.Request(requestCancelNotReadyForMedia, timespan);

                Console.WriteLine(respondingEvent.ToString());

                if (respondingEvent.Id == EventAck.MessageId)
                {
                    AgentReady = true;
                    return true;
                }
            }
            catch (ProtocolException ex)
            {
                Console.WriteLine(ex.ToString());
            }
            return false;
        }

        //Login Agent to a place
        public bool LoginAgent()
        {   // create request
            RequestAgentLogin requestAgentLogin =
                RequestAgentLogin.Create(
                tenantId,
                placeId,
                null);

            requestAgentLogin.AgentId = employeeId;

            Console.WriteLine(requestAgentLogin);
            // send request and receive response
            IMessage respondingEvent = itxServer.Request(requestAgentLogin, timespan);

            Console.WriteLine(respondingEvent);

            if (respondingEvent.Id == EventAck.MessageId)
            {
                AgentLoggedIn = true;
                return true;
            }

            return false;
        }

        //Logout Agent from a place
        public bool LogoutAgent()
        {
            RequestAgentLogout requestAgentLogout =
                RequestAgentLogout.Create();

            Console.WriteLine(requestAgentLogout);

            IMessage respondingEvent = itxServer.Request(requestAgentLogout, timespan);

            Console.WriteLine(respondingEvent);

            if (respondingEvent.Id == EventAck.MessageId)
            {
                AgentLoggedIn = false;
                AgentReady = false;
                return true;
            }

            return false;
        }
        
        //Add media to an Agent
        public bool AddMedia()
        {
            RequestAddMedia requestAddMedia =
                RequestAddMedia.Create();

            requestAddMedia.MediaTypeName = mediaType;

            Console.WriteLine(requestAddMedia);

            IMessage respondingEvent = itxServer.Request(requestAddMedia, timespan);

            Console.WriteLine(respondingEvent);

            if (respondingEvent.Id == EventAck.MessageId)
            {
                return true;
            }

            return false;
        }

        #endregion

        #region (4) Interaction handling. Accept and Stop.
        
        //Accept an incomming interaction
        public bool AcceptItx(int TicketId, string InteractionId)
        { 
            try
            {
                RequestAccept requestAccept =
                    RequestAccept.Create(
                    TicketId,
                    InteractionId);

                Console.WriteLine(requestAccept.ToString());
                
                IMessage respondingEvent = itxServer.Request(requestAccept, timespan);
                
                Console.WriteLine(respondingEvent.ToString());


                if (respondingEvent.Id == EventAck.MessageId)
                {
                    return true;
                }

            }
            catch (ProtocolException protocolException)
            {
                Console.WriteLine(protocolException.ToString());
            }
            return false;
        }

        //Stop Interaction
        public bool StopItx(string InteractionId)
        {
            try
            {
                RequestStopProcessing requestStopProcessing =
                    RequestStopProcessing.Create(
                    InteractionId,
                    null);

                Console.WriteLine(requestStopProcessing);
                
                IMessage respondingEvent = itxServer.Request(requestStopProcessing, timespan);
                
                Console.WriteLine(respondingEvent);
                
                if (respondingEvent.Id == EventAck.MessageId)
                {
                    return true;
                }
            }
            catch (ProtocolException protocolException)
            {
                Console.WriteLine(protocolException.ToString());
            }
            return false;
        }

        #endregion

        #region (5) Interaction server unsolicited messages receiver thread. 

        //Message receiver thread function
        public void ReceiveMessages()
        {
            try
            {
                while (itxServer.State == ChannelState.Opened)
                {
                    IMessage message = itxServer.Receive();
                    // Call a user-defined function that does the actual work:
                    if (message != null)
                    {
                        HandleMessages(message);
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return;
            }
        }

        // Message handler function
        public void HandleMessages(IMessage unsolicitedEvent)
        {
            try
            {
                switch (unsolicitedEvent.Id)
                {
                    case EventAgentLogin.MessageId:  //The event indicates that agent has logged in.  
                        doEventAgentLogin((EventAgentLogin)unsolicitedEvent);
                        break;

                    case EventAgentLogout.MessageId:  //The event indicates that agent has logged out.  
                        doEventAgentLogout((EventAgentLogout)unsolicitedEvent);
                        break;

                    case EventInvite.MessageId:  //  The event is sent to agent application to indicate that agent has been invited to participate in interaction processing. The event could be a result of routing of the interaction or transfer from or conference with another agent.  
                        doEventInvite((EventInvite)unsolicitedEvent);
                        break;

                    case EventCurrentAgentStatus.MessageId:  //The event is intended to provide current state of all medias resource can potentially handle to the resource. The message is generated by Stat Server and propagated to the client that represents the resource.  
                        doEventCurrentAgentStatus((EventCurrentAgentStatus)unsolicitedEvent);
                        break;

                    default:
                        Console.WriteLine(unsolicitedEvent.ToString());
                        break;
                }
            }
            catch (ProtocolException protocolException)
            {
                Console.WriteLine(protocolException.ToString());
            }
        }

        #endregion

        #region (6) Interaction server incoming unsolicited messages events handling. EventAgentLogin, EventAgentLogout, EventInvite, EventCurrentAgentStatus 
        //Process EventAgentLogin event
        private void doEventAgentLogin(EventAgentLogin eventAgentLogin)
        {
            Console.WriteLine(eventAgentLogin.ToString());
            AgentLoggedIn = true;
        }

        //Process EventAgentLogout event
        private void doEventAgentLogout(EventAgentLogout eventAgentLogout)
        {
            Console.WriteLine(eventAgentLogout.ToString());
            AgentLoggedIn = false;
        }

        //Process EventCurrentAgentStatus event
        private void doEventCurrentAgentStatus(EventCurrentAgentStatus eventCurrentAgentStatus)
        {
            Console.WriteLine(eventCurrentAgentStatus.ToString());

            if (eventCurrentAgentStatus.MediaStateList.Count > 0)
            {
                if ((Int32)eventCurrentAgentStatus.MediaStateList.AllValues[0] == 0)
                {
                    AgentReady = false;
                }
                else
                {
                    AgentReady = true; ;
                }
            }
        }

        //Process EventInvite event
        private void doEventInvite(EventInvite eventInvite)
        {
            Console.WriteLine(eventInvite.ToString());

            if(AcceptItx(eventInvite.TicketId, eventInvite.Interaction.InteractionId))
            {
                //Get data from event attached data
                String textData1 = eventInvite.Interaction.InteractionUserData.GetAsString("TextData1");
                String textData2 = eventInvite.Interaction.InteractionUserData.GetAsString("TextData2");

                //Show a message box with attached data
                String message = "TextData1 is : " + textData1 + "\nTextData2 is : " + textData2;
                MessageBox.Show(message);

                //Stop interaction
                StopItx(eventInvite.Interaction.InteractionId);
            }
        }

        #endregion

        #region (7) Application startup and console command input processing

        static void Main(string[] args)
        {
            Program AgentSample = new Program();

            Console.WriteLine("\nCurrect status:");
            Console.WriteLine("Agent logged in \t- " + AgentSample.AgentLoggedIn.ToString());
            Console.WriteLine("Agent ready \t\t- " + AgentSample.AgentReady.ToString());

            Console.WriteLine("\nPlease type:");
            Console.WriteLine("login \t\t- to login agent");
            Console.WriteLine("addmedia \t- to add media:");
            Console.WriteLine("ready \t\t- to set agent ready:");
            Console.WriteLine("logout \t\t- to logout agent");
            Console.WriteLine("exit \t\t- to exit");

            //Get application commands from console
            while (true)
            {
                String line = Console.ReadLine();
                switch (line)
                {
                    case "login":
                        AgentSample.LoginAgent();
                        break;
                    case "addmedia":
                        AgentSample.AddMedia();
                        break;
                    case "ready":
                        AgentSample.MakeAgentReady();
                        break;
                    case "logout":
                        AgentSample.LogoutAgent();
                        break;
                    case "exit":
                        if(AgentSample.DisonnectFromItxServer())
                            return;
                        break;
                    default:
                        Console.WriteLine("\nCurrect status:");
                        Console.WriteLine("Agent logged in \t- " + AgentSample.AgentLoggedIn.ToString());
                        Console.WriteLine("Agent ready \t\t- " + AgentSample.AgentReady.ToString());

                        Console.WriteLine("\nPlease type:");
                        Console.WriteLine("login \t\t- to login agent");
                        Console.WriteLine("addmedia \t- to add media:");
                        Console.WriteLine("ready \t\t- to set agent ready:");
                        Console.WriteLine("logout \t\t- to logout agent");
                        Console.WriteLine("exit \t\t- to exit");
                        break;
                }
            }
        }

        #endregion
    }
}
